package org.ten60.netkernel.visualizer;

import org.w3c.dom.*;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.util.XMLReadable;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class VisualizerViewAccessor extends StandardAccessorImpl
{
	public VisualizerViewAccessor()
	{	declareThreadSafe();
	}
	
	public void onSource(INKFRequestContext aHelper) throws Exception
	{
		INKFRequestReadOnly req=aHelper.getThisRequest();
		if (req.argumentExists("trace"))
		{	String traceId=req.getArgumentValue("trace");
			
			if (req.argumentExists("delete"))
			{	deleteTrace(aHelper,traceId);
				generateTracesView(aHelper);
			}
			else if (req.argumentExists("detail"))
			{	String requestId = req.getArgumentValue("detail");
				generateDetailView(aHelper,traceId,requestId);
			}
			else if (req.argumentExists("response"))
			{	String requestId = req.getArgumentValue("response");
				generateResponseView(aHelper,traceId,requestId);
			}
			else if (req.argumentExists("primary"))
			{	String requestId = req.getArgumentValue("primary");
				generatePrimaryView(aHelper,traceId,requestId);
			}
			else if (req.argumentExists("save"))
			{	generateSaveResponse(aHelper,traceId);
			}
			else
			{	generateTraceView(aHelper,traceId);
			}
		}
		else
		{	IHDSNode param = (IHDSNode)aHelper.source("httpRequest:/params",IHDSNode.class);
			Object action=param.getFirstValue("action2");
			if ("clear".equals(action))
			{	aHelper.source("active:visualizerClear");
			}
			else if ("start".equals(action))
			{	aHelper.source("active:visualizerStart");
			}
			else if ("stop".equals(action))
			{	aHelper.source("active:visualizerStop");
			}
			else if ("upload".equals(action))
			{	//String uploadedFilename=(String)param.getFirstValue("uploaded");
				IBinaryStreamRepresentation rep=(IBinaryStreamRepresentation)aHelper.source("httpRequest:/upload/uploaded",IBinaryStreamRepresentation.class);
				
				INKFRequest subreq=aHelper.createRequest("active:visualizerLoad");
				subreq.addArgumentByValue("data", rep);
				aHelper.issueRequest(subreq);
			}
			
			String matchString=(String)param.getFirstValue("match");
			if (matchString!=null)
			{	INKFRequest subreq=aHelper.createRequest("active:visualizerSetFilter");
				subreq.addArgumentByValue("data", matchString);
				aHelper.issueRequest(subreq);
			}
			generateTracesView(aHelper);
		}		
	}
	
	private void generateTracesView(INKFRequestContext aHelper) throws Exception
	{
		INKFRequest req = aHelper.createRequest("active:xsltc");
		req.addArgument("operator","res:/stylesheets/styleTraces.xsl");
		req.addArgument("operand","active:visualizerData");
		req.setRepresentationClass(aHelper.getThisRequest().getRepresentationClass());
		
		INKFResponseReadOnly subResp=aHelper.issueRequestForResponse(req);
		INKFResponse respOut=aHelper.createResponseFrom(subResp);
		respOut.setHeader("WrappedControlPanel", "Visualizer");
	}
	
	private void generateTraceView(INKFRequestContext aHelper, String aTraceId) throws Exception
	{
		INKFRequest subreq = aHelper.createRequestToEndpoint("visualizer");
		subreq.addArgument("action2", "Data");
		subreq.addArgument("trace", aTraceId);
		subreq.setRepresentationClass(Document.class);
		Document data = (Document)aHelper.issueRequest(subreq);

		INKFRequest req = aHelper.createRequest("active:xsltc");
		req.addArgument("operator","res:/stylesheets/styleTrace.xsl");
		req.addArgumentByValue("operand",data);
		req.setRepresentationClass(aHelper.getThisRequest().getRepresentationClass());
		INKFResponseReadOnly subResp=aHelper.issueRequestForResponse(req);

		INKFResponse resp=aHelper.createResponseFrom(subResp);
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
		
		XMLReadable r = new XMLReadable(data);
		String rootRequest=r.getText("/capture/trace/trace/identifier");
		resp.setHeader("WrappedControlPanel", "Visualizer - "+rootRequest);
	}
	
	private void generateDetailView(INKFRequestContext aHelper, String aTraceId, String aRequestId) throws Exception
	{	
		INKFRequest subreq = aHelper.createRequestToEndpoint("visualizer");
		subreq.addArgument("action2", "Data");
		subreq.addArgument("trace", aTraceId);
		subreq.addArgument("detail", aRequestId);
		subreq.setRepresentationClass(Document.class);
		Document data = (Document)aHelper.issueRequest(subreq);

		INKFRequest req2 = aHelper.createRequest("active:xslt");
		req2.addArgument("operator","res:/stylesheets/styleDetail.xsl");
		req2.addArgumentByValue("operand",data);
		req2.setRepresentationClass(aHelper.getThisRequest().getRepresentationClass());
		INKFResponseReadOnly subResp=aHelper.issueRequestForResponse(req2);

		INKFResponse resp=aHelper.createResponseFrom(subResp);
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);			

		XMLReadable r = new XMLReadable(data);
		String endpoint=r.getText("/detail/trace/endpointShort");		
		resp.setHeader("WrappedControlPanel", "Visualizer - "+endpoint);
	}
	
	private void generateResponseView(INKFRequestContext aHelper, String aTraceId, String aRequestId) throws Exception
	{ 
		INKFRequest subreq = aHelper.createRequestToEndpoint("visualizer");
		subreq.addArgument("action2", "Data");
		subreq.addArgument("trace", aTraceId);
		subreq.addArgument("response", aRequestId);
		INKFResponseReadOnly subResp = aHelper.issueRequestForResponse(subreq);
		aHelper.createResponseFrom(subResp);
	}
	
	private void generatePrimaryView(INKFRequestContext aHelper, String aTraceId, String aRequestId) throws Exception
	{ 
		INKFRequest subreq = aHelper.createRequestToEndpoint("visualizer");
		subreq.addArgument("action2", "Data");
		subreq.addArgument("trace", aTraceId);
		subreq.addArgument("primary", aRequestId);
		INKFResponseReadOnly subResp = aHelper.issueRequestForResponse(subreq);
		aHelper.createResponseFrom(subResp);
	}
	
	private void generateSaveResponse(INKFRequestContext aHelper, String aTraceId) throws Exception
	{
		INKFRequest subreq = aHelper.createRequestToEndpoint("visualizer");
		subreq.addArgument("action2", "Data");
		subreq.addArgument("trace", aTraceId);
		subreq.addArgument("save", "/save");
		INKFResponseReadOnly subResp = aHelper.issueRequestForResponse(subreq);
		
		aHelper.createResponseFrom(subResp);
		aHelper.sink("httpResponse:/header/Content-Disposition", "attachment; filename=netkernel_trace_"+aTraceId+".zip");
	}
	
	private void deleteTrace(INKFRequestContext aHelper, String aTraceId) throws Exception
	{
		INKFRequest subreq = aHelper.createRequestToEndpoint("visualizer");
		subreq.addArgument("action", "Delete");
		subreq.addArgument("data", aTraceId);
		aHelper.issueRequestForResponse(subreq);
	}
	
}
