<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="xml"/>
	<xsl:variable name="traces-view">/visualizer/</xsl:variable>
	<xsl:variable name="imgbase">/nkse/style/img/</xsl:variable>
	<xsl:param name="param"><sortBy>timestamp</sortBy></xsl:param>
	
	<xsl:template match="/trace" xmlns:xrl="http://netkernel.org/xrl">
		<div>
		
			<xrl:include>
			 <xrl:identifier>active:controlPanelInfoBox</xrl:identifier>
			 <xrl:argument name="content">
			   <div>
           <p>
             The Visualizer captures complete trace information about requests processed by NetKernel.
           </p>
           <p>
             The Visualizer captures a large quantity of information and is normally <i>off</i>
             to converse memory.
             To turn the Visualizer <i>on</i>, click the button labeled &quot;start&quot;.
             To turn the Visualizer <i>off</i>, click the button labeled &quot;stop&quot;.
             A regular expression filter may be entered to limit the requests captured
             while the Visualizer is on.
             The button labeled &quot;update&quot; will update the display to show all
             captured requests.
           </p>
           <p>
             To explore the details of a particular captured request,
             click on the root request identifier.
             This will display full details including each sub-request,
             returned representations and more.
					</p>
					<p>
            Refer to the
            <a href="/book/view/book:guide:logicalreference/doc:logicalreference:tools:visualizer">Visualizer Guide</a>
            for more details.
					</p>
			   </div>
			 </xrl:argument>
			</xrl:include> 
		
			<script language="Javascript"><xsl:comment>
				function doSubmit(action2)
				{	document.form.action2.value=action2;
					
					v=document.form.match.value;
					if (v=='Root request regex filter...')
					{	document.form.match.value='';
					}
					document.form.submit();
				}
			</xsl:comment></script>
				<form method="post" action="{$traces-view}" enctype="multipart/form-data" name="form">
					<input type="hidden" name="action2" value=""/>
					<table class="form-table">
						<tr>
							<td>
								<xsl:choose>
									<xsl:when test="enabled='true'">
										<input type="button" onclick="doSubmit('stop')" value="stop" class="btn"/>
									</xsl:when>
									<xsl:otherwise>
										<input type="button" onclick="doSubmit('start')" value="start" class="btn"/>
									</xsl:otherwise>
								</xsl:choose>
							</td><td>
								<input type="button" onclick="doSubmit('clear')" value="clear" class="btn"/>
							</td>
							<td>
								<input type="text" name="match" value="{match}">
									<xsl:choose>
										<xsl:when test="string-length(match)">
											<xsl:attribute name="value"><xsl:value-of select="match"/></xsl:attribute>
										</xsl:when>
										<xsl:otherwise>
											<xsl:attribute name="value">Root request regex filter...</xsl:attribute>
											<xsl:attribute name="onclick">value=''</xsl:attribute>
											<xsl:attribute name="onfocus">value=''</xsl:attribute>
											<xsl:attribute name="class">input-tip</xsl:attribute>
										</xsl:otherwise>
									</xsl:choose>
								</input>
							</td>
							<td>
								<input type="button" onclick="doSubmit('update')" value="update" class="btn"/>
							</td>
							<xsl:if test="impl!='org.netkernel.layer0.tools.EndpointMonitor'">
								<td>
									Import Saved Trace State:
								</td><td>
									<input type="file" name="uploaded"/>
								</td>
								<td>
									<input type="button" onclick="doSubmit('upload')" value="upload"  class="btn"/>
								</td>
							</xsl:if>
						</tr>
					</table>
				</form>
				<table class="data-table">
					<tr class="vis-title">
						<td>Root Request Identifier
							<xsl:call-template name="sortButtons">
								<xsl:with-param name="key" select="'id'"/>
							</xsl:call-template>
						</td>
						<td>Timestamp
							<xsl:call-template name="sortButtons">
								<xsl:with-param name="key" select="'timestamp'"/>
							</xsl:call-template>							
						</td>
						<td>Duration
							<xsl:call-template name="sortButtons">
								<xsl:with-param name="key" select="'duration'"/>
							</xsl:call-template>							
						</td>
						<td/>
					</tr>
					<xsl:choose>
						<xsl:when test="$param = 'timestamp'">
							<xsl:apply-templates select="captured/trace">
								<xsl:sort select="timestamp" order="ascending"/>
							</xsl:apply-templates>
						</xsl:when>
						<xsl:when test="$param = 'timestampRev'">
							<xsl:apply-templates select="captured/trace">
								<xsl:sort select="timestamp" order="descending"/>
							</xsl:apply-templates>
						</xsl:when>
						<xsl:when test="$param = 'id'">
							<xsl:apply-templates select="captured/trace">
								<xsl:sort select="request" order="ascending"/>
							</xsl:apply-templates>
						</xsl:when>
						<xsl:when test="$param = 'idRev'">
							<xsl:apply-templates select="captured/trace">
								<xsl:sort select="request" order="descending"/>
							</xsl:apply-templates>
						</xsl:when>
						<xsl:when test="$param = 'duration'">
							<xsl:apply-templates select="captured/trace">
								<xsl:sort select="duration" order="ascending"/>
							</xsl:apply-templates>
						</xsl:when>
						<xsl:when test="$param = 'durationRev'">
							<xsl:apply-templates select="captured/trace">
								<xsl:sort select="duration" order="descending"/>
							</xsl:apply-templates>
						</xsl:when>
					</xsl:choose>
				</table>
		</div>		
	</xsl:template>
	
	<xsl:template match="trace">
		<xsl:variable name="class">
			<xsl:choose>
				<xsl:when test="selected">
					<xsl:text>selected</xsl:text>
				</xsl:when>
				<xsl:when test="position() mod 2 = 0">
					<xsl:text>evenrow</xsl:text>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>oddrow</xsl:text>
				</xsl:otherwise>  
			</xsl:choose>
		</xsl:variable>
		<tr class="{$class}">
			<td><a href="{$traces-view}{hash}"><xsl:value-of select="request"/></a></td>
			<td><xsl:value-of select="timestamp"/></td>
			<td align="right"><xsl:value-of select="duration"/></td>
			<td><a href="{$traces-view}{hash}/delete"><img border="0" src="{$imgbase}delete.png"/></a></td>
		</tr>
	</xsl:template>
	
	<xsl:template name="sortButtons">
		<xsl:param name="key"/>
	
		<xsl:choose>
			<xsl:when test="$param=$key">
					<a href="{$traces-view}?sortby={$key}"><img class="sort" src="{$imgbase}upSelected.png" alt="[^]"/></a>
			</xsl:when>
			<xsl:otherwise>
				<a href="{$traces-view}?sortby={$key}"><img class="sort" src="{$imgbase}up.png" alt="[^]"/></a>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:choose>
			<xsl:when test="$param=concat($key,'Rev')">
					<a href="{$traces-view}?sortby={$key}Rev"><img class="sort" src="{$imgbase}downSelected.png" alt="[v]"/></a>
			</xsl:when>
			<xsl:otherwise>
				<a href="{$traces-view}?sortby={$key}Rev"><img class="sort" src="{$imgbase}down.png" alt="[v]"/></a>
			</xsl:otherwise>
		</xsl:choose>
	
	</xsl:template>
	
	
</xsl:stylesheet>
